%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% File name: RPEMStatic.m
%
% Description: MIMO RPEM algorithm for identification of 
%              nonlinear static systems. Include noise covariance
%              estimation.
%
% Inputs: Input signals, dimensions and output signals. 
%
% Outputs: Estimated parameters.
%
% Author and copyright: Torbjrn Wigren
%                       Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       Uppsala
%                       Sweden
%
% Revision log: Created 2005-08-23.
%
% Firm revision: revision 2.0, 2005-08-23.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% Dimensions
% 

%
% Generate indices to sum over
%

if ~exist('theta_0_new')
   'Input polynomialOrders and notUsedIndices'
   keyboard;
   allIndices=GenerateIndices(polynomialOrders,notUsedIndices)
   dimTheta=size(allIndices,1);
   'Input theta_0'
	 keyboard;
end
if exist('theta_0_new')
   allIndices=GenerateIndices(polynomialOrders,notUsedIndices);
   dimTheta=size(allIndices,1);
   theta_0=theta_0_new;
end

%
% Remaining initial values
%

Gamma_0=initialNoiseVariance*eye(size(y,1),size(y,1));
R_0=scaleFactorR*eye(dimTheta,dimTheta);

% Storage variables

y_m_save=zeros(ny,ceil(N/downSampling)+1);
perr_save=zeros(ny,ceil(N/downSampling));
theta_save=zeros(dimTheta,ceil(N/downSampling)+1);
eig_R_save=zeros(dimTheta,ceil(N/downSampling)+1);
condition_save=zeros(1,ceil(N/downSampling)+1);

%
% Algorithmic loop
%

% Initiation at t=0...

mu=mu_0;
y_m=y_m_0;
Gamma=Gamma_0;
x_m=x_m_0;
R=R_0;
theta=theta_0;

y_m_save(:,1)=y_m_0;
theta_save(:,1)=theta_0;

percentReady=0;

for t=1:N
   
   if round(t*100/N)>percentReady
      percentReady=round(t*100/N)
   end
   
   %
   % Calculate mu(t)
   %
   
   mu=mu0*mu+1-mu0;
     
   %
   % Prediction error and Gamma update
   %
   
   perr=y(:,t)-y_m;
  	Gamma=Gamma+(mu/(muFactor+t))*(perr*perr'-Gamma);
   
   %
   % Covariance update
   %
   
   R=R+(mu/(muFactor+t))*(dhdthetaStatic_m(ny,x_m,u(:,t),theta,allIndices)'*inv(Gamma)*dhdthetaStatic_m(ny,x_m,u(:,t),theta,allIndices)-R);
   
   %
   % Parameter update
   %
   
	theta=theta+(mu/(muFactor+t))*inv(R)*dhdthetaStatic_m(ny,x_m,u(:,t),theta,allIndices)'*inv(Gamma)*perr;
      
   %
   % Model output computation
   %
   
   if t<N
   	y_m=hStatic_m(ny,x_m,u(:,t+1),theta,allIndices);
   end   
      
   %
   % Check and save
   %
   
   if rem(t,downSampling)==0 % Time to save
      y_m_save(:,t/downSampling+1)=y_m;
      perr_save(:,t/downSampling)=perr;
      theta_save(:,t/downSampling+1)=theta;
      eig_R_save(:,t/downSampling+1)=eig(R);
      condition_save(:,t/downSampling+1)=cond(R);
   end
   
end

reInitiateTheta=theta;
reInitiateIndices=allIndices;

%
% Compute the re-scaled, original parameter vector
%

scalingTs=1;
rescaledTheta=theta;
for i=1:dimTheta
   for j=1:nx
      thisSubIndex=allIndices(i,j);
      rescaledTheta(i,1)=rescaledTheta(i,1)*((1/scalingTs)^(thisSubIndex*(j-1)));
   end
end
rescaledTheta=rescaledTheta*(scalingTs)^nx;

%
% End of algorithmic loop
%

[theta rescaledTheta]

